import java.io.*;
import java.util.*;

/**
  This class is a container for all the information that is transferred from the
  Wumpus World Server to the Agent Application. It contains the percepts, image percept,
  natural language hint, current step number, maximum number of steps per trial,
  current trial number, maximum trials per game, avatar information(IP address and
  port number), current score, previous action taken,  number of arrows remaining,
  current status of the agent (active, defeated, victorious),timeout limit.

  A VERY IMPORTANT NOTE:
  This class definition is shared by both Wumpus World Server and Agent Application. So if
  this file is changed( by even a white space), then this should be replaced in the Agent Application
  otherwise the Wumpus World Server and Agent Application will not be able to communicate using
  the instance of this class.

 * @author Ashwini Kuntamukkala
 * @version 	%I%, %G%
 * @since       JDK1.2

*/

public class ConInfo implements Serializable
{
   private boolean stench;
   private boolean breeze;
   private boolean glitter;
   private boolean bump;
   private boolean scream;
   private int actionTaken;
   private int score;
   private double averageScore;
   private double stdDeviation;
   private int numArrow;
   private int numGold;
   private int status;
   private int[][][] imagePixels;
   private int imageHeight;
   private int imageWidth;
   private String hint;

   private boolean defeated;

   /*Newly added attributes*/
   private String avataarIP;
   private int avataarPort;
   private int clientID;
   private String msg;
   private int maxSteps;
   private int noofTrials;
   private int currentStep;
   private int currentTrial;
   private int timeoutLimit;

  /**
  Sets the maximum number of steps allowed per trial.

  @param  Maximum number of steps per trial.
  */

   public void setMaxStepsPerTrial(int mS)
   {
	   maxSteps = mS;
   }

   /**
   Returns the maximum number of steps per trial.

   @return Maximum number of steps per trial.
   */

   public int getMaxSteps()
   {
	   return maxSteps;
   }

   /**
   Sets the maximum number of trials per game.

   @param  Maximum number of trials per game.
   */
   public void setNumberOfTrials(int nT)
   {
	   noofTrials = nT;
   }

   /**
   Returns the maximum number of trials per game.

   @return Maximum number of trials per game.
   */

   public int getNumberOfTrials()
   {
	   return noofTrials;
   }

   /**
   Sets the current step number.

   @param  The current step number.
   */

   public void setCurrentStep(int cS)
   {
	   currentStep = cS;
   }

   /**
   Sets the timeout limit for the connection.
   This is used when the agents are handled in a round robin fashion.
   Every agent is served after all the agents who obtained connection before it.
   So the agent has to wait for the Wumpus World Server to send the percepts. This wait
   however cannot be indefinitely long. The Wumpus World Server, thus sends a pessimistic
   upperbound on time that an agent may have to wait to be served. If the agent does not
   get served during that time, then the agent can infer that it has been disconnected.

   @param  Timeout Limit.
   */

   public void setTimeoutLimit(int x)
   {
	   timeoutLimit = x;
   }

   /**
   Returns the timeout limit.

   @return Timeout Limit.
   */

   public int getTimeoutLimit()
   {
	   return timeoutLimit;
   }

   /**
   Returns the current step number.

   @return current step number.
   */

   public int getCurrentStep()
   {
	   return currentStep;
   }

   /**
   Sets the current trial number.

   @param Current Trial Number.
   */

   public void setCurrentTrial(int cT)
   {
	   currentTrial = cT;
   }

   /**
   Returns the current trial number.

   @return current trial number.
   */

   public int getCurrentTrial()
   {
	   return currentTrial;
   }


   /**
   Sets the message that should be sent to the agent.

   @param The Message to be sent.
   */

   public void setMsg(String x)
   {
	   msg = x;
   }

   /**
   Returns the message.

   @return Message.
   */

   public String getMsg()
   {
	   return msg;
   }

  /**
 	Sets the port number of the avatar application.
    This is the port number where the avatar application is waiting to accept connection
    from the corresponding agent.
   @param The port number.
  */

   public void setPort(int x)
   {
	   avataarPort = x;
   }


  /**
 	Sets the IP address of the avatar application.
    This is the IP address of the avatar to which the corresponding agent attempts to
    obtain a connection to.
   @param The IP address.
  */

   public void setIP(String x)
   {
	   avataarIP = x;
   }


  /**
 	Sets the ID number of the agent.

   @param The ID of the agent.
  */

   public void setID(int x)
   {
	   clientID = x;
   }

   /**
  	Returns the IP address of the avatar application.
    @return IP address.
   */

   public String getIP()
   {
	   return avataarIP;
   }

   /**
  	Returns the port number the avatar application.
    @return Port number.
   */
   public int getPort()
   {
	   return avataarPort;
   }

   /**
  	Returns the ID of the agent.
    @return ID.
   */

   public int getID()
   {
	   return clientID;
   }


   /**
   Constructor that takes no arguments.
   */

   public ConInfo()
   {
    stench = breeze = glitter = bump = scream = false;
    defeated = false;

   }

   /**
  	Returns the status of the agent.
    @return status of the agent(active, defeated, victorious).
   */

   public int getStatus()
   {
	   return status; //defeated;
   }

   /**
  	Sets the percepts
  	@param stench, breeze, glitter, bump

   */

   public void setPercept(boolean st, boolean br, boolean gli, boolean bu)
   {
    stench = st;
    breeze = br;
    glitter = gli;
    bump = bu;
    //scream = sc;
   }

   /**
  	Sets the scream percept
    @param scream
   */

    public void setScream(boolean sc)
    {
		scream = sc;
	}

   /**
  	Sets the action taken
    @param action taken
   */

   public void setActionTaken(int a)
   {
    actionTaken = a;
   }

   /**
  	Sets the score of agent
    @param score
   */

   public void setScore(int a)
   {
    score = a;
   }

   /**
	Returns the scream percept
    @return score
   */

   public int getScore()
   {
    return score;
   }

   /**
  	Sets the number of arrows remaining
    @param number of arrows
   */

   public void setNumArrow(int a)
   {
    numArrow = a;
   }

   /**
	Returns the number of arrows
    @return number of arrows
   */

   public int getNumArrow()
   {
    return numArrow;
   }

   /**
  	Sets the number of gold treasures the agent collected
    @param number of gold treasures
   */

   public void setNumGold(int a)
   {
    numGold = a;
   }

   /**
  	Sets the natural language hint
    @param natural language hint
   */

   public void setHint(String s)
   {
    hint = s;
   }

   /**
  	Sets the image percept of the cell the agent is facing
    @param a three dimension array of integers, height and width of image in pixels
   */

   public void setImage(int[][][] arr, int height, int width)
   {
    imageHeight = height;
    imageWidth = width;
    imagePixels = new int[height][width][3];

    for(int k = 0; k<3; ++k)
     for(int i =0; i< height; ++i)
      {
       for(int j = 0; j<width; ++j)
       {
        imagePixels[i][j][k] = arr[i][j][k];
       }
      }
   }

   /**
  	Sets the average score
    @param average score
   */

   public void setAverageScore(double a)
   {
    averageScore = a;
   }

   /**
  	Sets the standard deviation
    @param standard deviation
   */

   public void setStdDeviation(double a)
   {
    stdDeviation = a;
   }

   /**
  	Returns the image width
    @return image width
   */

   public int getImageWidth()
   {
    return imageWidth;
   }

   /**
  	Returns the image height
    @return image height
   */

   public int getImageHeight()
   {
    return imageHeight;
   }

   /**
  	Returns the image pixels.
  	@return Image Pixels.
   */

   public int[][][] getImagePixels()
   {
    return imagePixels;
   }

   /**
  	Returns the number of treasures of gold collected.
    @return number of gold treasures collected.
   */

   public int getNumGold()
   {
    return numGold;
   }

   /**
  	Sets the status of the agent
    @param status
   */

   public void setStatus(int a)
   {
    status = a;
   }

   /**
  	Returns the action taken
    @return action taken in previous action
   */

   public int getActionTaken()
   {
    return actionTaken;
   }

   /**
  	Returns the stench percept
    @return stench
   */
   public boolean getStench()
   {
    return stench;
   }

   /**
  	Returns the breeze percept
    @return breeze
   */
   public boolean getBreeze()
   {
    return breeze;
   }

   /**
  	Returns the glitter percept
    @return glitter
   */

   public boolean getGlitter()
   {
    return glitter;
   }

   /**
  	Returns the bump   percept
    @return bump
   */
   public boolean getBump()
   {
    return bump;
   }

   /**
  	Returns the scream percept
    @return scream
   */
   public boolean getScream()
   {
    return scream;
   }
   /**
  	Returns the natural language hint
    @return natural language hint
   */

   public String getHint()
   {
    return hint;
   }

   /**
  	Returns the average score
    @return average score
   */
   public double getAverageScore()
   {
    return averageScore;
   }

   /**
  	Returns the standard deviation
    @return standard deviation
   */
   public double getStdDeviation()
   {
    return stdDeviation;
   }

}

