import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class MainContentPanel extends JPanel {
    private Dimension size; // indicates size taken up by graphics
    private CustomizedPanel drawingArea;
    private JToolBar    myToolbar;
	private	JToggleButton		buttonPit;
	private	JToggleButton		buttonObstacle;
	private	JToggleButton		buttonGold;
	private	JToggleButton		buttonWumpus;
	private JToggleButton       buttonArrow;
	private JToggleButton       buttonUndo;

	private	JPanel 				instructionPanel;

	private int 				buttonCount;

	public void setButtonsEnabled(boolean status)
	{
		buttonPit.setEnabled(status);
		buttonObstacle.setEnabled(status);
		buttonGold.setEnabled(status);
		buttonWumpus.setEnabled(status);
		buttonArrow.setEnabled(status);
		buttonUndo.setEnabled(status);
	}
    public MainContentPanel() {
        setOpaque(true);
        size = new Dimension(0,0);
        instructionPanel = new JPanel(new GridLayout(0,15));
		JLabel statusBar = new JLabel();
		/* Added to incorporate the wumpus thread */
		Environment env = new Environment();
        drawingArea = new CustomizedPanel(env, statusBar, "plan.txt", "obstacle.txt");
		env.setCustPanel(drawingArea);
        drawingArea.setBackground(Color.black);
        JScrollPane scroller = new JScrollPane(drawingArea);
        scroller.setPreferredSize(new Dimension(600,600));

		buttonCount = 1;
		buttonPit = addToolbarButton( instructionPanel, true, null,
								"pit.gif", "Place a pit" );
		buttonObstacle = addToolbarButton( instructionPanel, true, null,
								"Obstacle.gif", "Place an obstacle" );
		buttonGold = addToolbarButton( instructionPanel, true, null,
								"pot_o_gold.gif", "Place gold" );
		buttonWumpus = addToolbarButton( instructionPanel, true, null,
								"wumpusFront.gif", "Place a wumpus and set mode" );
		buttonArrow = addToolbarButton(instructionPanel, true, null, "mousecursor.gif", "Set default cursor");

		buttonUndo = addToolbarButton(instructionPanel, true, null, "UNDO.gif", "Undo previous action");


        //Layout this demo.
        setLayout(new BorderLayout());
        add(instructionPanel, BorderLayout.NORTH);
        add(scroller, BorderLayout.CENTER);
        add(statusBar, BorderLayout.SOUTH);
    }

    public CustomizedPanel getDrawingArea()
    {
		return drawingArea;
	}
	public void readWorld(String s)
	{
		drawingArea.readWorld(s);
	}
	public void writeWorld(String s)
	{
		System.out.println(s);
		drawingArea.writeWorld(s);
	}
    public void setToggleButton(JToggleButton btn)
    {
		if(buttonPit!=btn)
			buttonPit.setSelected(false);

		if(buttonObstacle!=btn)
			buttonObstacle.setSelected(false);
		if(buttonGold!=btn)
			buttonGold.setSelected(false);
		if(buttonWumpus!=btn)
			buttonWumpus.setSelected(false);

		buttonArrow.setSelected(false);
		buttonUndo.setSelected(false);
	}
	public JToggleButton addToolbarButton( JPanel toolBar, boolean bUseImage, String sButtonText,
									String sButton, String sToolHelp )
	{
		JToggleButton b;

		// Create a new button
		if( bUseImage )
		    {
			b = new JToggleButton	( new ImageIcon( sButton));
		    }
		else
			b = (JToggleButton)toolBar.add( new JToggleButton());

		b.setBounds(5,buttonCount*30,30,30);
		buttonCount++;
		// Add the button to the toolbar
		toolBar.add( b );

		// Add optional button text
		if( sButtonText != null )
			b.setText( sButtonText );
		else
		{
			// Only a graphic, so make the button smaller
			b.setMargin( new Insets( 0, 0, 0, 0 ) );
		}


		// Add optional tooltip help
		if( sToolHelp != null )
			b.setToolTipText( sToolHelp );

		// Make sure this button sends a message when the user clicks it
		b.setActionCommand( "Toolbar:" + sButton );

		b.addActionListener( new ToolBarHandler() );

		return b;
	}

	private class ToolBarHandler implements ActionListener
	{

		public void actionPerformed(ActionEvent e)
		{
			setToggleButton((JToggleButton)e.getSource());
			if(e.getActionCommand().equals("Toolbar:pit.gif"))
			{
				drawingArea.setPitCursor();
			}
			else
			if(e.getActionCommand().equals("Toolbar:Obstacle.gif"))
			 {
				drawingArea.setObstacleCursor();
			 }
			else
			if(e.getActionCommand().equals("Toolbar:pot_o_gold.gif"))
			  {
				  drawingArea.setGoldCursor();
			  }
			else
			if(e.getActionCommand().equals("Toolbar:wumpusFront.gif"))
			   {
				  drawingArea.setWumpusCursor();
			   }
			else
			if(e.getActionCommand().equals("Toolbar:mousecursor.gif"))
			   {
				   drawingArea.setDefaultCursor();
			   }
		    else
		    if(e.getActionCommand().equals("Toolbar:UNDO.gif"))
		    {
				drawingArea.undoPreviousAction();
			}

		}
	}
	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		drawingArea.setPreferredSize(new Dimension(2532,2532));
		drawingArea.scrollRectToVisible(new Rectangle(0,0,2532,2532));
		drawingArea.revalidate();
		drawingArea.draw();
	}
   public void showToolbar(boolean show)
   {
	   if(show)
	   {
		   instructionPanel.setVisible(true);
	   }
	   else
	   {
		   instructionPanel.setVisible(false);
	   }
   }
}

